﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TuiBase
{
    public class ComboBox<ValueType> : Control
    {
        public ComboBox(string caption, int captionLength)
        {
            Caption = caption;
            _captionLength = captionLength;
            _boxLength = Math.Max(0, Size.X - captionLength - 2);
        }

        public ComboBox(string caption)
            : this(caption, caption.Length)
        {
        }

        public string Caption { get; private set; }
        private int _captionLength;
        private int _boxLength;


        public IList<ComboBoxItem<ValueType>> Items { get; set; }

        private ValueType _selectedValue;

        public ValueType SelectedValue
        {
            get
            {
                return _selectedValue;
            }
            set
            {
                _selectedValue = value;
                ComboBoxItem<ValueType> item = Items.First(i => object.Equals(i.Value, value));
                if (item != null)
                    Text = item.Text;
            }
        }


        protected override void OnSizeChanged()
        {
            _boxLength = Math.Max(0, Size.X - _captionLength - 2);
        }

        protected override void OnTextChanged()
        {
            _boxLength = Math.Max(0, Size.X - _captionLength - 2);
        }

        protected override void OnActivate()
        {
            base.OnActivate();
            Console.SetCursorPosition(Location.X + _captionLength+1 , Location.Y);
            Console.CursorVisible = true;
        }

        protected override void OnDeactivate()
        {
            base.OnDeactivate();
            Console.CursorVisible = false;
        }


        protected override void OnPaint()
        {
            ushort color = ConsoleColor.Attr(Foreground, Background);
            uint w;

            Coord pos = new Coord(Location.X, Location.Y);

            string s = "▐";
            if (!string.IsNullOrEmpty(Caption))
            {
                s = (Caption.Length > _captionLength ? Caption.Substring(0, _captionLength) : Caption.PadRight(_captionLength)) + s;
            }
            Cons.WriteConsoleOutputCharacterEncoded( s, (uint)s.Length, pos, out w);
            Cons.FillConsoleOutputAttribute(Cons.ConsoleHandle, color, (uint)s.Length, pos, out w);

            WriteBoxText(null);

            pos = new Coord((short)(Location.X + Size.X - 1), Location.Y);
            Cons.WriteConsoleOutputCharacterEncoded("▼", 1, pos, out w);
            Cons.FillConsoleOutputAttribute(Cons.ConsoleHandle, ConsoleColor.Attr(Background,Foreground), 1, pos, out w);


        }

        private void WriteBoxText(string t)
        {
            ushort color = ConsoleColor.Attr(Foreground, Background);
            uint w;

            string text = t ?? Text ?? string.Empty;
            text = text.Length > _boxLength ? text.Substring(0, _boxLength) : text.PadRight(_boxLength);

            Coord pos = new Coord((short)(Location.X + _captionLength + 1), Location.Y);
            Cons.WriteConsoleOutputCharacterEncoded( text, (uint)text.Length, pos, out w);
            Cons.FillConsoleOutputAttribute(Cons.ConsoleHandle,ConsoleColor.Attr(Background,Foreground), (uint)text.Length, pos, out w);

        }


        #region Funktionalität


        public Action SelectionChanged { get; set; }

        protected override bool OnKeyPress(ConsoleKeyInfo keyInfo)
        {
            if (base.OnKeyPress(keyInfo))
                return true;
            else
            {
                if (keyInfo.Key == ConsoleKey.DownArrow)
                {
                    PopUpMenu popup = new PopUpMenu(string.Empty, Items.Select(i => i.Text).ToArray());
                    popup.Location = Location + new Coord(_captionLength, 1);
                    popup.Size = new Coord(Math.Max(5, _boxLength + 2), Math.Max(5, Math.Min(10, Items.Count + 2)));
                    if (popup.ShowDialog() == DialogResultType.Ok)
                    {
                        SelectedValue = Items[popup.SelectedItem].Value;
                        Text = Items[popup.SelectedItem].Text;
                        WriteBoxText(Text);
                        if (Items[popup.SelectedItem].Action != null)
                            Items[popup.SelectedItem].Action();
                        if (SelectionChanged != null)
                            SelectionChanged();
                    }

                    return true;
                }
                else
                    return false;
            }
        }
        #endregion
    }
}
